@@ -148,7 +148,11 @@ class Package(object):
         :rtype: str
         :raises: :exc:`InvalidPackage`
         """
-        build_type_exports = [e.content for e in self.exports if e.tagname == 'build_type']
+        # for backward compatibility a build type without an evaluated
+        # condition is still being considered (i.e. evaluated_condition is None)
+        build_type_exports = [
+            e.content for e in self.exports
+            if e.tagname == 'build_type' and e.evaluated_condition is not False]
         if not build_type_exports:
             return 'catkin'
         if len(build_type_exports) == 1:
@@ -196,6 +200,7 @@ class Package(object):
             'replaces',
             'group_depends',
             'member_of_groups',
+            'exports',
         ):
             conditionals = getattr(self, attr)
             for conditional in conditionals:
@@ -293,7 +298,8 @@ class Package(object):
                 new_warnings.append('Metapackage "%s" must buildtool_depend on catkin.' % self.name)
             if self.has_invalid_metapackage_dependencies():
                 new_warnings.append('Metapackage "%s" should not have other dependencies besides a '
-                                    'buildtool_depend on catkin and run_depends.' % self.name)
+                                    'buildtool_depend on catkin and %s.' %
+                                    (self.name, 'run_depends' if self.package_format == 1 else 'exec_depends'))
 
         for warning in new_warnings:
             if warnings is None:
@@ -352,12 +358,13 @@ class Dependency(object):
 
 
 class Export(object):
-    __slots__ = ['tagname', 'attributes', 'content']
+    __slots__ = ['tagname', 'attributes', 'content', 'evaluated_condition']
 
     def __init__(self, tagname, content=None):
         self.tagname = tagname
         self.attributes = {}
         self.content = content
+        self.evaluated_condition = None
 
     def __str__(self):
         txt = '<%s' % self.tagname
@@ -369,6 +376,21 @@ class Export(object):
             txt += '/>'
         return txt
 
+    def evaluate_condition(self, context):
+        """
+        Evaluate the condition.
+
+        The result is also stored in the member variable `evaluated_condition`.
+
+        :param context: A dictionary with key value pairs to replace variables
+          starting with $ in the condition.
+
+        :returns: True if the condition evaluates to True, else False
+        :raises: :exc:`ValueError` if the condition fails to parse
+        """
+        self.evaluated_condition = evaluate_condition(self.attributes.get('condition'), context)
+        return self.evaluated_condition
+
 
 # Subclassing ``str`` to keep backward compatibility.
 class License(str):
@@ -512,6 +534,8 @@ def parse_package_string(data, filename=None, warnings=None):
     :returns: return parsed :class:`Package`
     :raises: :exc:`InvalidPackage`
     """
+    if sys.version_info[0] == 2 and not isinstance(data, str):
+        data = data.encode('utf-8')
     try:
         root = dom.parseString(data)
     except Exception as ex:
@@ -675,7 +699,11 @@ def parse_package_string(data, filename=None, warnings=None):
     nodes = [n for n in root.childNodes if n.nodeType == n.ELEMENT_NODE]
     unknown_tags = set([n.tagName for n in nodes if n.tagName not in known.keys()])
     if unknown_tags:
-        errors.append('The manifest (with format version %d) must not contain the following tags: %s' % (pkg.package_format, ', '.join(unknown_tags)))
+        errors.append('The manifest of package "%s" (with format version %d) must not contain the following tags: %s' % (pkg.name, pkg.package_format, ', '.join(unknown_tags)))
+    if 'run_depend' in unknown_tags and pkg.package_format >= 2:
+        errors.append('Please replace <run_depend> tags with <exec_depend> tags.')
+    elif 'exec_depend' in unknown_tags and pkg.package_format < 2:
+        errors.append('Either update to a newer format or replace <exec_depend> tags with <run_depend> tags.')
     for node in [n for n in nodes if n.tagName in known.keys()]:
         errors += _check_known_attributes(node, known[node.tagName])
         if node.tagName not in ['description', 'export']:
